/*****************************************************************************
 * Copyright (c) 2020, rrcSmall FM10K-Documentation Contributors
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the copyright holder nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#pragma once

#include <string>
#include <vector>

class Configuration {
public:

    Configuration() {

    }

    class ConfigurationNode {
    public:

        enum class Type {
            Node = 0,
            ValueText,
            ValueInt,
            ValueBool,
        } type;

        ConfigurationNode(std::string name, ConfigurationNode::Type type = ConfigurationNode::Type::Node) : name(
                std::move(name)), type(type) {

        }

        std::string name;
        std::vector<ConfigurationNode> possibleNodes;
        std::string value;

        std::string getTypeString() const {
            switch (type) {
                case Type::Node:
                    return "node";
                case Type::ValueText:
                    return "text";
                case Type::ValueInt:
                    return "int";
                case Type::ValueBool:
                    return "bool";
                default:
                    return "unknown";
            }
        }

        const ConfigurationNode *getSubNode(const std::string &k) const;

        ConfigurationNode &getOrCreateSubNode(const std::string &k);

        bool getBool() const {
            return value == "true" || value == "1";
        }

        int getInteger() const {
            return std::stoi(value, nullptr, 0);
        }

        std::pair<uint32_t, uint32_t> getEUI64ToInteger() const;

        const std::string &getText() const {
            return value;
        }
    };

    ConfigurationNode mainNode = ConfigurationNode("");
    std::vector<std::string> orderedNodes;

    std::vector<std::string> getAllEntries() const;

    void getAllEntriesForNode(std::vector<std::string> &entries, const ConfigurationNode &node,
                              const std::string &key) const;

    const ConfigurationNode &getEntry(const std::string &entry) const;

    bool addEntry(const std::string &entry);

};