/*****************************************************************************
 * Copyright (c) 2020, rrcSmall FM10K-Documentation Contributors
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the copyright holder nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#pragma once


#include <cstdint>
#include <unordered_map>
#include <vector>
#include <algorithm>
#include "ImageFormat.h"

class OutputContext {
public:

private:
    const ImageFormat &image;
    std::unordered_map<uint32_t, std::string> mappings;
    std::unordered_map<uint32_t, std::string> comments;
    std::unordered_map<uint32_t, std::string> registers;

public:

    OutputContext(const ImageFormat &image) : image(image) {

    }

    void addMapping(uint32_t location, const std::string &name, bool force = false);

    void addComment(uint32_t location, const std::string &comment);

    void addRegister(uint32_t addr, const std::string &name) {
        registers[addr] = name;
    }

    std::string getAddressRegisterName(uint32_t addr, uint8_t offset) const;

    std::string getAddressRegisterName(const Instruction::AddressWithOffset &address) const {
        return getAddressRegisterName(address.address, address.offset);
    }

    std::string getLabel(uint32_t location, bool force = true) const;

    std::string getComment(uint32_t location) const;

    std::string getDataHeader(uint32_t location) const;

    std::string getDataEntry(uint32_t location, const std::string &representation = "") const;

    std::string getInstructionString(const Instruction::Instruction &instruction) const;

    void analyze();
};
