/*****************************************************************************
 * Copyright (c) 2020, rrcSmall FM10K-Documentation Contributors
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the copyright holder nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#pragma once


#include <cstdint>
#include <string>
#include "instructions/AddressOffset.h"

enum class KnownRegisters : uint32_t {
    PCIE_MASTER_SPICO_FIRMWARE_SPECIAL_REGISTER = 0x0,
    FATAL_CODE = 0x0,
    PCIE_BROADCAST_SERDES_FIRMWARE_SPECIAL_REGISTER = 0x1,
    LAST_FATAL_CODE = 0x1,

    SOFT_RESET = 0x3,
    DEVICE_CFG = 0x4,
    RESET_CFG = 0x5,
    WATCHDOG_CFG = 0x6,

    MGMT_SCRATCH_0 = 0x8,
    MGMT_SCRATCH_1 = MGMT_SCRATCH_0 + 1,

    VITAL_PRODUCT_DATA = 0x304,

    GLOBAL_INTERRUPT_DETECT = 0x400,

    INTERRUPT_MASK_BSM = 0x442,

    SRAM_ERR_IM_0 = 0x448,
    SRAM_ERR_IM_1 = 0x449,

    SW_IM = 0x44E,

    CHIP_VERSION = 0x452,

    BSM_SCRATCH_START = 0x800,
    BSM_SCRATCH_END = BSM_SCRATCH_START + 0x400 - 1,

    BSM_CTRL = 0x000C00,
    BSM_ARGS = 0x000C01,

    BSM_ADDR_OFFSET_0 = 0x000C04,
    BSM_ADDR_OFFSET_1 = BSM_ADDR_OFFSET_0 + 1,
    BSM_ADDR_OFFSET_2 = BSM_ADDR_OFFSET_0 + 2,
    BSM_ADDR_OFFSET_3 = BSM_ADDR_OFFSET_0 + 3,

    BSM_COUNTER_0 = 0x000C08,
    BSM_COUNTER_1 = BSM_COUNTER_0 + 1,

    BIST_CTRL_0 = 0x000C10,
    BIST_CTRL_1 = 0x000C11,

    REI_CTRL = 0x000C12,
    REI_STAT = 0x000C13,

    GPIO_CFG = 0x000C15,
    GPIO_DATA = 0x000C16,
    GPIO_IP = 0x000C17,
    GPIO_IM = 0x000C18,

    PLL_PCIE_CTRL = 0x2241,
    PLL_PCIE_STAT = 0x2242,
    SBUS_PCIE_CFG = 0x2243,
    SBUS_PCIE_COMMAND = 0x2244,
    SBUS_PCIE_REQUEST = 0x2245,
    SBUS_PCIE_RESPONSE = 0x2246,
    SBUS_PCIE_SPICO_IN = 0x2247,
    SBUS_PCIE_SPICO_OUT = 0x2248,
    SBUS_PCIE_IP = 0x2249,
    SBUS_PCIE_IM = 0x224a,

    PCIE_XPLL_CTRL = 0x3000,
    PCIE_CLK_CTRL = 0x3001,
    PCIE_CLK_CTRL_2 = 0x3002,
    PCIE_CLKMON_RATIO_CFG = 0x3003,
    PCIE_CLKMON_TOLERANCE_CFG = 0x3004,
    PCIE_CLKMON_DEADLINES_CFG = 0x3005,
    PCIE_CLK_STAT = 0x3006,
    PCIE_CLK_IP = 0x3007,
    PCIE_CLK_IM = 0x3008,
    PCIE_WARM_RESET_DELAY = 0x3009,

    PORTS_MGMT_BASE_ADDRESS = 0xE8000,
    PLL_EPL_CTRL = PORTS_MGMT_BASE_ADDRESS + 0x0,
    PLL_EPL_STAT = PORTS_MGMT_BASE_ADDRESS + 0x1,
    PLL_FABRIC_CTRL = PORTS_MGMT_BASE_ADDRESS + 0x2,
    PLL_FABRIC_STAT = PORTS_MGMT_BASE_ADDRESS + 0x3,
    PLL_FABRIC_LOCK = PORTS_MGMT_BASE_ADDRESS + 0x4,
    SBUS_EPL_CFG = PORTS_MGMT_BASE_ADDRESS + 0x5,
    SBUS_EPL_COMMAND = PORTS_MGMT_BASE_ADDRESS + 0x6,
    SBUS_EPL_REQUEST = PORTS_MGMT_BASE_ADDRESS + 0x7,
    SBUS_EPL_RESPONSE = PORTS_MGMT_BASE_ADDRESS + 0x8,
    SBUS_EPL_SPICO_IN = PORTS_MGMT_BASE_ADDRESS + 0x9,
    SBUS_EPL_SPICO_OUT = PORTS_MGMT_BASE_ADDRESS + 0xa,
    SBUS_EPL_IP = PORTS_MGMT_BASE_ADDRESS + 0xb,
    SBUS_EPL_IM = PORTS_MGMT_BASE_ADDRESS + 0xc,

    PM_CLKOBS_CTRL = PORTS_MGMT_BASE_ADDRESS + 0x12,

    PCIE_PF_BASE_ADDRESS = 0x100000,

    NOP = 0xFFFFFFFF
};

KnownRegisters getScratchRegister(uint32_t offset);

std::string getRegisterName(KnownRegisters addr);

std::string getAddressRegisterName(uint32_t addr, uint8_t offset = 0);

std::string getAddressRegisterName(const Instruction::AddressWithOffset &address);