/*****************************************************************************
 * Copyright (c) 2020, rrcSmall FM10K-Documentation Contributors
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the copyright holder nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#pragma once

#include <cstdint>
#include <string>
#include <vector>
#include <unordered_map>
#include <memory>
#include "../AnalysisState.h"

class OutputContext;

namespace Instruction {
    typedef struct {
        uint32_t location;
        std::string format;
    } OutputFormat;

    class Instruction {
    protected:
        uint32_t _address{};
        uint32_t _endAddress{};
    public:

        enum class CommandOp : uint8_t {
            WRITE = 0b00000000,
            COPY = 0b01000000,
            LOAD = 0b11000000,
            INIT = 0b11001000,
            CALC = 0b10000000,
            CALC_IMM = 0b10100000,
            BRANCH = 0b11010000,
            POLL = 0b11011000,
            LOOP = 0b11100000,
            JUMP = 0b11101000,
            RETURN = 0b11110000,
            SET = 0b11111000,
            WAIT = 0xFE,
            END = 0xFF
        };

        static std::unique_ptr<Instruction>
        decodeInstructionFromBytes(uint32_t offset, const std::vector<uint8_t> &bytes);

        static CommandOp getCommandFromByte(uint8_t command);

        virtual void fromBytes(uint32_t offset, const std::vector<uint8_t> &bytes) = 0;

        virtual std::vector<uint8_t> toBytes() const = 0;

        virtual std::string toString() const = 0;

        virtual std::vector<OutputFormat> toOutputFormat(const OutputContext &context) const = 0;

        virtual std::vector<uint32_t> getPossibleBranches() const = 0;

        virtual std::vector<std::pair<uint32_t, std::unordered_map<uint32_t, uint32_t>>>
        execute(AnalysisState &state) const = 0;

        virtual CommandOp getCommand() const = 0;

        uint32_t getAddress() const {
            return _address;
        }

        uint32_t getEndAddress() const {
            return _endAddress;
        }

    };
};
