/*****************************************************************************
 * Copyright (c) 2020, rrcSmall FM10K-Documentation Contributors
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the copyright holder nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

#include "Return.h"
#include "../Registers.h"
#include <iostream>
#include <iomanip>
#include "../OutputContext.h"

void Instruction::Return::fromBytes(uint32_t offset, const std::vector<uint8_t> &bytes) {
    _address = offset;

    uint8_t command = bytes[offset++];

    address.offset = command & 0b11;

    address.address = bytes[offset++] << 16;
    address.address |= bytes[offset++] << 8;
    address.address |= bytes[offset++];


    _endAddress = offset;
}

std::string Instruction::Return::toString() const {
    std::stringstream op;
    op << "RETURN " << getAddressRegisterName(address);

    return op.str();
}

std::vector<Instruction::OutputFormat> Instruction::Return::toOutputFormat(const OutputContext &context) const {
    return std::vector<OutputFormat>{
            OutputFormat{getAddress(), dynamic_cast<std::stringstream &>(std::stringstream("") << "RETURN" << " ["
                                                                                               << context.getAddressRegisterName(
                                                                                                       address)
                                                                                               << "]").str()}
    };
}

std::vector<uint32_t> Instruction::Return::getPossibleBranches() const {
    return std::vector<uint32_t>();
}

std::vector<std::pair<uint32_t, std::unordered_map<uint32_t, uint32_t>>>
Instruction::Return::execute(AnalysisState &state) const {
    uint32_t jumpAddress = state.getRegister(address);
    state.current = jumpAddress;

    state.addKnownJump(jumpAddress, getAddress(), JumpKind::Return);

    return std::vector<std::pair<uint32_t, std::unordered_map<uint32_t, uint32_t>>>();
}


std::vector<uint8_t> Instruction::Return::toBytes() const {
    std::vector<uint8_t> bytes;

    bytes.emplace_back((uint8_t) getCommand() | address.offset);

    bytes.emplace_back((address.address >> 16) & 0xFF);
    bytes.emplace_back((address.address >> 8) & 0xFF);
    bytes.emplace_back(address.address & 0xFF);

    return bytes;
}